@@ -7,7 +7,8 @@ import traceback
 from pathlib import Path
 
 from netmiko import ConnectHandler
-from netmiko import NetmikoAuthenticationException, SSHDetect
+from netmiko import NetmikoAuthenticationException
+from netmiko.exceptions import SSHException
 from netmiko.ssh_autodetect import SSHDetect
 import requests
 import yaml
@@ -118,6 +119,8 @@ class dyagram:
                 raise
             self.pbar.update(self.pbar_update_int)
             self.log.info(f"DEVICE: {device} - RESTCONF : Discovering LLDP Neighbors - SUCCESSFUL")
+        except SSHException:
+            print("CAUGHT SSH EXCEPT 02")
         except:
             try:
             # try ssh
@@ -158,8 +161,8 @@ class dyagram:
                     self.log.info(f"DEVICE: {device} - Submitted for discover_lldp_neighbors")
                     executor.submit(self.discover_routes, device)
                     self.log.info(f"DEVICE: {device} - Submitted for discover_routes")
-                    executor.submit(self.discover_dynamic_routing_neighbors, device)
-                    self.log.info(f"DEVICE: {device} - Submitted for discover_dynamic_routing_neighbors")
+                    # executor.submit(self.discover_dynamic_routing_neighbors, device)
+                    # self.log.info(f"DEVICE: {device} - Submitted for discover_dynamic_routing_neighbors")
 
                     self._devices_queried.append(device)
 
@@ -187,6 +190,8 @@ class dyagram:
                 print("\nChanges in state!")
             elif not self.changes_in_state and self.state_exists:
                 print("\nNo changes in state")
+        except SSHException:
+            print("CAUGHT SSH EXCEPT 01")
         except:
             try:
                 tb = self.get_traceback()
@@ -229,6 +234,8 @@ class dyagram:
                 raise
             self.pbar.update(self.pbar_update_int)
             self.log.info(f"DEVICE: {device} - RESTCONF : Discovering routes - SUCCESSFUL")
+        except SSHException:
+            print("CAUGHT SSH EXCEPT 04")
         except:
             try:
                 self.log.info(f"DEVICE: {device} - RESTCONF : Discovering routes - FAILURE")
@@ -369,6 +376,8 @@ class dyagram:
             # self.discover_bgp_neighbors(device)
 
             self.pbar.update(self.pbar_update_int)
+        except SSHException:
+            print("CAUGHT SSH EXCEPT 03")
         except:
             tb = self.get_traceback()
             self.log.info(f"DEVICE: {device} EXCEPTION THROWN IN discover_dynamic_routing_neighbors: {tb}")
@@ -408,31 +417,37 @@ class dyagram:
         return None
 
     def discover_eigrp_neighbors_ssh(self, device):
-        os = self.get_device_type(device)
-        if os:
-            nm_args = self.netmiko_args_template.copy()
-            nm_args['host'] = device
-            nm_args['device_type'] = os
 
-        else:
-            raise Exception("Unable to determine OS.")
+        try:
+            os = self.get_device_type(device)
+            if os:
+                nm_args = self.netmiko_args_template.copy()
+                nm_args['host'] = device
+                nm_args['device_type'] = os
+                print(f"NETMIKO ARGS FROM COPY TEMPLATE: {nm_args}")
+
+            else:
+                raise Exception("Unable to determine OS.")
 
 
-        dev = ConnectHandler(**nm_args)
-        dev.enable()
-        if os in ['cisco_ios', 'cisco_nxos', 'cisco_xr']:
-            eigrp_output = dev.send_command("show ip eigrp neighbors vrf all") #textfsm not currently supported for this command
-            if isinstance(eigrp_output, str):
-                neighbor_ips = re.findall('\d+\.\d+\.\d+\.\d+', eigrp_output)
-                self.log.info(f"{device}: EIGRP NEIGHBOR IPs - {neighbor_ips}")
-            for i in self.topology['devices']:
-                if i['inventory_ip'] == device:
-                    i['dynamic_routing_neighbors']['eigrp'] = neighbor_ips
-                    break
-        else:
+            dev = ConnectHandler(**nm_args)
+            dev.enable()
+            if os in ['cisco_ios', 'cisco_nxos', 'cisco_xr']:
+                eigrp_output = dev.send_command("show ip eigrp neighbors vrf all") #textfsm not currently supported for this command
+                if isinstance(eigrp_output, str):
+                    neighbor_ips = re.findall('\d+\.\d+\.\d+\.\d+', eigrp_output)
+                    self.log.info(f"{device}: EIGRP NEIGHBOR IPs - {neighbor_ips}")
+                for i in self.topology['devices']:
+                    if i['inventory_ip'] == device:
+                        i['dynamic_routing_neighbors']['eigrp'] = neighbor_ips
+                        break
+            else:
+                dev.disconnect()
+                raise Exception(f"OS {os} Not supported")
             dev.disconnect()
-            raise Exception(f"OS {os} Not supported")
-        dev.disconnect()
+        except:
+            tb = self.get_traceback()
+            self.log.info(f"DEVICE: {device} - SSH : discover_eigrp_neighbors_ssh - {tb}")
 
 
 
